/*
 * Copyright 2004 JTracker.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jtracker.jtcase.eng;

import java.util.Vector;
import java.util.HashMap;
import java.util.Map;
import org.jtracker.jtcode.framework.model.file.FileModel;
import org.jtracker.jtcode.framework.model.file.LineModel;

public class TPIDAnalyser extends FileModel{
  final static String TPID_SIGN_COMMENTMODE_BEGIN="//";
  final static int TPID_SIGN_COMMENTMODE_LONG_MIN = 2;
  final static String TPID_SIGN_TPMODE_BEGIN = "/*<<<";
  final static String TPID_SIGN_TPMODE_SET = "=";
  final static String TPID_SIGN_TPMODE_END = ">>>*/";
  final static int TPID_SIGN_TPMODE_LONG_MIN = 11;
  final static String SRC_PACKAGE = "package";
  final static String SRC_LINE_END = ";";
  final static String SRC_FILE_DIV = ".";
  final static String SRC_DIR_DIV = "\\";
  final static String SRC_FUNC_START = "";
  final static String SRC_TESTPOINT = "";
  final static String SRC_DESCRIP_DIV = "|  ";
  final static int SRC_LINE_NUMBER_DIV = 4;
  final static int SRC_DESCRIP_LENGTH = 5;
  Vector TpIDLst = new Vector();
  Vector dubIDLst = new Vector();
  Map idMap = new HashMap();
  String path="";
  String packageName ="";
  String className="";
  String srcFile = "";
  String idFilter = "";
  StringBuffer srcBuffer = new StringBuffer();
  boolean hasPackage=false;
  public TPIDAnalyser() {
  }
  public TPIDAnalyser(String fileName,String filter){
  	idFilter = filter;
    produceInfoFromFile(fileName);
    TpIDLst = new Vector();
    processFile(fileName);
  }
  public Vector getIDfromSource(String fileName){
    produceInfoFromFile(fileName);
    TpIDLst = new Vector();
    processFile(fileName);
    return TpIDLst;
  }
  public Vector getTpIDLst(){
    return TpIDLst;
  }
  public Vector getdubIDLst(){
    return dubIDLst;
  }
  public String getPackageName(){
    return packageName;
  }
  public String getClassName(){
    return className;
  }
  public String getSrcFile(){
  	return srcFile;
  }
  public String getPath(){
    return path;
  }
  protected void processLine(String str,int line){
  String sourceText = fromatLength(""+line,"_",SRC_LINE_NUMBER_DIV);
  TPIDModel idObj = new TPIDModel(str,line);
  String id = idObj.getTpID();
  String idStr = idObj.getTpIDItem();
  if(MethodModel.isMethodDefine(str)==true) {
	String methodName = MethodModel.getMethodNameByString(str);
	TPIDModel obj = new TPIDModel();
	obj.setTpID(methodName);
	obj.setLine(line);
	TpIDLst.add(obj);
	sourceText+=SRC_FUNC_START;
	}
  if(idStr!=null) {//1
  	if((idFilter==null||idFilter.equals(""))){//2
  		sourceText+=SRC_TESTPOINT;
		TpIDLst.add(idObj);
			if(idMap.containsKey(id)){dubIDLst.add(idObj);
			   }else{idMap.put(id,new Integer(line));}
  	     }else if(idStr.indexOf(idFilter)>=0){//2-1
		    sourceText+=SRC_TESTPOINT;
			TpIDLst.add(idObj);
				if(idMap.containsKey(id)){dubIDLst.add(idObj);
				   }else{idMap.put(id,new Integer(line));}
				}//2-1 	       
  	     }//1-1

  //look for package info
  if(!hasPackage){
    if(str.indexOf(SRC_PACKAGE)>=0){packageName = producePackageFromSrc(str);}
    }
//    sourceText= fromatLength(sourceText,"_",SRC_DESCRIP_LENGTH);
    sourceText+=SRC_DESCRIP_DIV;
    sourceText+=str;
	srcBuffer.append(sourceText+"\n");
  }
private String producePackageFromSrc(String str){
    String id = "";
    int idx_s = str.indexOf(SRC_PACKAGE);
    int idx_e = str.indexOf(SRC_LINE_END);
    id = str.substring(idx_s+SRC_PACKAGE.length(),idx_e);
    return id.trim();
}
private void produceInfoFromFile(String str){
    int idx_f = str.lastIndexOf(SRC_DIR_DIV);
    if(idx_f<0)idx_f=0;
    int idx_e = str.lastIndexOf(SRC_FILE_DIV);
    srcFile = str.substring(idx_f+1,str.length());
    className = str.substring(idx_f+1,idx_e);
    path=str.substring(0,idx_f);
    setFileName(str);
}
private String fromatLength(String str,String replace,int reqLength){
	String strRet=str;
	int  size=reqLength-str.length();
	if(size<=0)return str;
	for(int i=0;i<size;i++){
	strRet+=replace;
	}
	return strRet;
}
public static void main(String args[]){
TPIDAnalyser obj = new TPIDAnalyser(args[0],"");
Vector vLst = obj.getTpIDLst();
System.out.println("***************************************");
System.out.println("==============ID LIST==================");
for(int i=0;i<vLst.size();i++){
System.out.println(((TPIDModel)vLst.get(i)).getTpIDItem());
}
System.out.println("***************************************");
System.out.println("==============dubID LIST==================");
Vector vLst2 = obj.getdubIDLst();
for(int i=0;i<vLst2.size();i++){
System.out.println(((TPIDModel)vLst2.get(i)).getTpIDItem());
}
System.out.println("***************************************");
System.out.println("package:"+obj.getPackageName());
System.out.println("class:"+obj.getClassName());
}
/**
 * @return
 */
public String getIdFilter() {
	return idFilter;
}

/**
 * @param string
 */
public void setIdFilter(String string) {
	idFilter = string;
}
public TPIDModel getTPID(int idx){
  TPIDModel idModel = (TPIDModel)TpIDLst.get(idx);
  return idModel;
}
/**
 * @return
 */
public String getSrcText() {
	return srcBuffer.toString();
}

}